/*
    This file is part of the CLib sub-project of the FreeDOS project
    Copyright (C) 1997 by the author see below

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* $RCSfile: MEMREV.C $
   $Locker:  $	$Name:  $	$State: Exp $

	char_t *memrev(char_t *str, size_t length)

	Reverse memory area.

	Input:
		str != NULL

	Return:
		str

	Note:
		length is treated unsigned

	Conforms to:
		<none>

	See also:
		strrev

	Target compilers:
		Any C compiler

	Origin:
		1997/10/10 Steffen Kaiser (ska)

	Revised by:
		<none>

	File Revision:    Revision 1.2  1998/02/08 04:55:59  ska
*/

#include <_clib.h>			/* standard include, must be the first! */
#include <_string.h>

#ifdef RCS_Version
static char const rcsid[] = 
	"$Id: MEMREV.C 1.2 1998/02/08 04:55:59 ska Exp $";
#endif

/* Method:
	One pointer to the very first byte, another to the very last one;
	the moving both to the middle until they reach. */
_CLibFunc char_t ___PTR *memrev(char_t ___PTR * const s, size_t length)
{	REG char_t ___PTR *p, ___PTR *q, c;

	assert(s != NULL);

	/* maybe length < 2; s+length points one byte behind the area
		thus q is to be pre-decremented */
	for(q = length + (p = s); p < --q; ++p) {
		/* swap the characters */
		c = *p;
		*p = *q;
		*q = c;
	}

	return s;
}
